@@ -793,6 +793,16 @@ public:
         auto xdg_surface = std::make_shared<XdgSurfaceStable>(client, surface);
         auto xdg_toplevel = std::make_shared<XdgToplevelStable>(*xdg_surface);
 
+        // Protocol *requires* us to ack the initial configure before attaching a buffer
+        bool initial_configure_received = false;
+        EXPECT_CALL(*xdg_surface, configure(testing::_))
+            .WillOnce(
+                [&initial_configure_received, xdg_surface = static_cast<struct xdg_surface*>(*xdg_surface)](uint32_t serial)
+                {
+                    xdg_surface_ack_configure(xdg_surface, serial);
+                    initial_configure_received = true;
+                });
+
         surface.run_on_destruction([xdg_surface, xdg_toplevel]() mutable
             {
                 xdg_toplevel.reset();
@@ -801,6 +811,23 @@ public:
 
         wl_surface_commit(surface);
 
+        client.dispatch_until([&initial_configure_received]() { return initial_configure_received; });
+        /* Make absolutely sure the functor above, which captures a stack variable by reference
+         * is not going to be invoked outside this function
+         */
+        testing::Mock::VerifyAndClearExpectations(xdg_surface.get());
+
+        /* And we might as well ack any subsequent configures.
+         * Since we don't need to capture anything but the xdg_surface, which has to be
+         * live if we're getting a callback on it, this is safe to escape this frame
+         */
+        ON_CALL(*xdg_surface, configure(testing::_))
+            .WillByDefault(
+                [xdg_surface = static_cast<struct xdg_surface*>(*xdg_surface)](uint32_t serial)
+                {
+                    xdg_surface_ack_configure(xdg_surface, serial);
+                });
+
         surface.attach_visible_buffer(width, height);
 
         return surface;
@@ -878,8 +905,6 @@ public:
 
     wl_surface* window_under_cursor() const
     {
-        if (pointer_events_pending())
-            BOOST_THROW_EXCEPTION(std::runtime_error("Pointer events pending"));
         if (current_pointer_location)
         {
             return current_pointer_location->surface;
@@ -889,8 +914,6 @@ public:
 
     wl_surface* touched_window() const
     {
-        if (touch_events_pending())
-            BOOST_THROW_EXCEPTION(std::runtime_error("Touch events pending"));
         wl_surface* surface = nullptr;
         for (auto const& touch : current_touches)
         {
@@ -904,15 +927,11 @@ public:
 
     std::pair<wl_fixed_t, wl_fixed_t> pointer_position() const
     {
-        if (pointer_events_pending())
-            BOOST_THROW_EXCEPTION(std::runtime_error("Pointer events pending"));
         return current_pointer_location.value().coordinates;
     };
 
     std::pair<wl_fixed_t, wl_fixed_t> touch_position() const
     {
-        if (touch_events_pending())
-            BOOST_THROW_EXCEPTION(std::runtime_error("Touch events pending"));
         if (current_touches.empty())
             BOOST_THROW_EXCEPTION(std::runtime_error("No touches"));
         else if (current_touches.size() == 1)
@@ -2196,3 +2215,7 @@ void wlcs::ShmBuffer::add_release_listener(std::function<bool()> const &on_relea
 {
     impl->add_release_listener(on_release);
 }
+
+wlcs::OutputState::OutputState(wl_output* output) : output{output}
+{
+}
